@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6calc.c
- * Version    : $Id: ipv6calc.c,v 1.18 2003/08/28 18:51:55 peter Exp $
+ * Version    : $Id: ipv6calc.c,v 1.19 2005/09/15 12:14:00 peter Exp $
  * Copyright  : 2001-2003 by Peter Bieringer <pb (at) bieringer.de>
  * 
  * Information:
@@ -771,7 +771,7 @@ int main(int argc,char *argv[]) {
 
 	
 	/* clear resultstring */
-	snprintf(resultstring, sizeof(resultstring), "%s", "");
+	snprintf(resultstring, sizeof(resultstring) - 1, "%s", "");
 	
 	switch (action) {
 		case ACTION_mac_to_eui64:
@@ -969,7 +969,7 @@ int main(int argc,char *argv[]) {
 			retval = libipv6addr_ipv6addrstruct_to_tokenlsb64(&ipv6addr2, resultstring3, formatoptions);
 			
 			/* cat together */
-			snprintf(resultstring, sizeof(resultstring), "%s %s", resultstring2, resultstring3);
+			snprintf(resultstring, sizeof(resultstring) - 1, "%s %s", resultstring2, resultstring3);
 			break;
 
 
