@@ -1,8 +1,8 @@
 /*
  * Project    : ipv6calc
  * File       : ipv6logconv.c
- * Version    : $Id: ipv6logconv.c,v 1.6 2003/11/21 09:36:57 peter Exp $
- * Copyright  : 2002 by Peter Bieringer <pb (at) bieringer.de>
+ * Version    : $Id: ipv6logconv.c,v 1.15 2005/09/15 17:48:32 peter Exp $
+ * Copyright  : 2002-2005 by Peter Bieringer <pb (at) bieringer.de>
  * 
  * Information:
  *  Dedicated program for logfile conversions
@@ -40,6 +40,7 @@
 
 long int ipv6calc_debug = 0;
 int flag_quiet = 0;
+int flag_nocache = 0;
 
 /* supported output types:
  *  ipv6addr
@@ -51,6 +52,20 @@ int flag_quiet = 0;
 static int converttoken(char *result, const char *token, const long int outputtype, const int flag_skipunknown);
 static void lineparser(const long int outputtype);
 
+
+/* LRU cache */
+
+#define CACHE_LRU_SIZE 200
+
+static int      cache_lru_max = 0;
+static int      cache_lru_last = 0;
+int      cache_lru_limit = 20; /* optimum */
+static char     cache_lru_key_token[CACHE_LRU_SIZE][NI_MAXHOST];
+static long int cache_lru_key_outputtype[CACHE_LRU_SIZE];
+static char     cache_lru_value[CACHE_LRU_SIZE][NI_MAXHOST];
+static long int cache_lru_statistics[CACHE_LRU_SIZE];
+
+
 /**************************************************/
 /* main */
 #define DEBUG_function_name "ipv6logconv/main"
@@ -60,7 +75,7 @@ int main(int argc,char *argv[]) {
 
 
 	/* new option style storage */	
-	int inputtype = -1, outputtype = -1;
+	uint32_t inputtype  = FORMAT_undefined, outputtype = FORMAT_undefined;
 	
 	/* convert storage */
 	long int action = -1;
@@ -102,18 +117,33 @@ int main(int argc,char *argv[]) {
 				ipv6calc_debug = atol(optarg);
 				break;
 
+			case 'c':
+				cache_lru_limit = atoi(optarg);
+				if (cache_lru_limit > CACHE_LRU_SIZE) {
+					cache_lru_limit = CACHE_LRU_SIZE;
+					fprintf(stderr, " Cache limit too big, built-in limit: %d\n", cache_lru_limit);
+				};
+				if (cache_lru_limit < 1) {
+					cache_lru_limit = 1;
+					fprintf(stderr, " Cache limit too small, take minimum: %d\n", cache_lru_limit);
+				};
+				break;
+
+			case 'n':
+				flag_nocache = 1;
+				break;
+
 			case CMD_outputtype:
 				if (ipv6calc_debug != 0) {
 					fprintf(stderr, "%s: Got output string: %s\n", DEBUG_function_name, optarg);
 				};
 				if ( (strcmp(optarg, "-?") == 0) || (strcmp(optarg, "-h") == 0) || (strcmp(optarg, "--help") == 0) ) {
-					outputtype = -2;
-					command = CMD_printhelp;
+					command = CMD_printexamples;
 					break;
 				};
 				
 				outputtype = ipv6calctypes_checktype(optarg);
-				if (outputtype < 0) {
+				if (outputtype == FORMAT_undefined) {
 					fprintf(stderr, " Output option is unknown: %s\n", optarg);
 					exit(EXIT_FAILURE);
 				};
@@ -134,10 +164,8 @@ int main(int argc,char *argv[]) {
 
 	/* print help handling */
 	if (command == CMD_printhelp) {
-		if (outputtype == -2) {
-			printhelp_outputtypes(FORMAT_ipv6logconv);
-			exit(EXIT_FAILURE);
-		};
+		ipv6logconv_printhelp();
+		exit(EXIT_FAILURE);
         } else if (command == CMD_printexamples) {
 		printhelp_output_dispatcher(outputtype);
 		exit(EXIT_FAILURE);
@@ -153,12 +181,6 @@ int main(int argc,char *argv[]) {
 		exit(EXIT_FAILURE);
 	};
 
-	if (command == CMD_printhelp) {
-		ipv6logconv_printhelp();
-		exit(EXIT_FAILURE);
-	};
-	
-
 	/* call lineparser */
 	lineparser(outputtype);
 
@@ -176,7 +198,7 @@ static void lineparser(const long int outputtype) {
 	char token[LINEBUFFER];
 	char resultstring[LINEBUFFER];
 	char *charptr, *cptr, **ptrptr;
-	int linecounter = 0, retval;
+	int linecounter = 0, retval, i;
 
 	ptrptr = &cptr;
 	
@@ -232,7 +254,7 @@ static void lineparser(const long int outputtype) {
 			continue;
 		};
 
-		snprintf(token, sizeof(token), "%s", charptr);
+		snprintf(token, sizeof(token) - 1, "%s", charptr);
 		
 		if (ipv6calc_debug != 0) {
 			fprintf(stderr, "%s: Token 1: '%s'\n", DEBUG_function_name, token);
@@ -307,6 +329,13 @@ static void lineparser(const long int outputtype) {
 
 	if (flag_quiet == 0) {
 		fprintf(stderr, "...finished\n");
+
+		if (flag_nocache == 0) {
+			fprintf(stderr, "Cache statistics:\n");
+			for (i = 0; i < cache_lru_limit; i++) {
+				fprintf(stderr, "Cache distance: %3d  hits: %8ld\n", i, cache_lru_statistics[i]);
+			};
+		};
 	};
 	return;
 };
@@ -320,8 +349,9 @@ static void lineparser(const long int outputtype) {
 static int converttoken(char *resultstring, const char *token, const long int outputtype, const int flag_skipunknown) {
 	long int inputtype = -1;
 	int retval = 1, i;
-	int typeinfo, typeinfo_test;
+	uint32_t typeinfo, typeinfo_test;
 	char tempstring[NI_MAXHOST], temp2string[NI_MAXHOST];
+	ipv6calc_macaddr macaddr;
 
 	/* used structures */
 	ipv6calc_ipv6addr ipv6addr;
@@ -338,6 +368,56 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 		return (1);
 	};
 
+	/* use cache ? */
+	if (flag_nocache == 0 && cache_lru_max > 0) {
+		/* check last seen one first */
+		if (ipv6calc_debug & 0x4) {
+			fprintf(stderr, "LRU cache: look for key=%s\n", token);
+		};
+
+		if (cache_lru_key_outputtype[cache_lru_last - 1] == outputtype) {
+			if (strcmp(cache_lru_key_token[cache_lru_last - 1], token) == 0) {
+				snprintf(resultstring, LINEBUFFER - 1, cache_lru_value[cache_lru_last - 1]);
+				cache_lru_statistics[0]++;
+				if (ipv6calc_debug & 0x4) {
+					fprintf(stderr, "LRU cache: hit last line=%d key_token=%s key_outputtype=%lx value=%s\n", cache_lru_last - 1, token, outputtype, resultstring);
+				};
+				return (0);
+			};
+		} else {
+			/* run backwards to first entry */
+			if (cache_lru_last > 1) {
+				for (i = cache_lru_last - 1; i > 0; i--) {
+					if (cache_lru_key_outputtype[i - 1] == outputtype) {
+						if (strcmp(cache_lru_key_token[i - 1], token) == 0) {
+							snprintf(resultstring, LINEBUFFER - 1, cache_lru_value[i - 1]);
+							cache_lru_statistics[cache_lru_last - i]++;
+							if (ipv6calc_debug & 0x4) {
+								fprintf(stderr, "LRU cache: hit line=%d key_token=%s key_outputtype=%lx value=%s\n", i - 1, token, outputtype, resultstring);
+							};
+							return (0);
+						};
+					};
+				};
+			};
+			/* round robin */ 
+			if (cache_lru_last < cache_lru_max) {
+				for (i = cache_lru_max; i > cache_lru_last; i--) {
+					if (cache_lru_key_outputtype[i - 1] == outputtype) {
+						if (strcmp(cache_lru_key_token[i - 1], token) == 0) {
+							snprintf(resultstring, LINEBUFFER - 1, cache_lru_value[i - 1]);
+							cache_lru_statistics[cache_lru_max - i + cache_lru_last]++;
+							if (ipv6calc_debug & 0x4) {
+								fprintf(stderr, "LRU cache: hit line=%d key_token=%s key_outputtype=%lx value=%s\n", i - 1, token, outputtype, resultstring);
+							};
+							return (0);
+						};
+					};
+				};
+			};
+		};
+	};
+
 
 	/* set addresses to invalid */
 	ipv6addr.flag_valid = 0;
@@ -384,7 +464,7 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 	switch (outputtype) {
 		case FORMAT_addrtype:
 			if (ipv6addr.flag_valid == 1) {
-				snprintf(resultstring, NI_MAXHOST, "ipv6-addr.addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "ipv6-addr.addrtype.ipv6calc");
 
 				/* check for registry */
 				typeinfo = ipv6addr_gettype(&ipv6addr);
@@ -397,7 +477,7 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 						break;
 					};
 				};
-				typeinfo_test = typeinfo & (IPV6_NEW_ADDR_AGU | IPV6_ADDR_LINKLOCAL | IPV6_ADDR_SITELOCAL | IPV6_ADDR_MAPPED | IPV6_ADDR_COMPATv4);
+				typeinfo_test = typeinfo & (IPV6_NEW_ADDR_AGU | IPV6_ADDR_LINKLOCAL | IPV6_ADDR_SITELOCAL | IPV6_ADDR_MAPPED | IPV6_ADDR_COMPATv4 | IPV6_ADDR_ULUA);
 				if ( typeinfo_test != 0 ) {
 					/* get string */
 					for (i = 0; i < (size_t) (sizeof(ipv6calc_ipv6addrtypestrings) / sizeof(ipv6calc_ipv6addrtypestrings[0])); i++) {
@@ -407,8 +487,8 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 						};
 					};
 				};
-				snprintf(tempstring, sizeof(tempstring), "%s.%s", ipv6calc_ipv6addrtypestrings[retval].token, resultstring);
-				snprintf(resultstring, NI_MAXHOST,"%s", tempstring);
+				snprintf(tempstring, sizeof(tempstring) - 1, "%s.%s", ipv6calc_ipv6addrtypestrings[retval].token, resultstring);
+				snprintf(resultstring, LINEBUFFER - 1,"%s", tempstring);
 
 				/* type of global IPv6 address */
 				/* init retval */
@@ -427,16 +507,16 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 							break;
 						};
 					};
-					snprintf(tempstring, sizeof(tempstring), "%s.%s", ipv6calc_ipv6addrtypestrings[retval].token, resultstring);
-					snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+					snprintf(tempstring, sizeof(tempstring) - 1, "%s.%s", ipv6calc_ipv6addrtypestrings[retval].token, resultstring);
+					snprintf(resultstring, LINEBUFFER - 1, "%s", tempstring);
 				};
 
 				/* registry of IPv6 address */
 				if ( (typeinfo & (IPV6_NEW_ADDR_6BONE | IPV6_NEW_ADDR_PRODUCTIVE) ) != 0 ) {
 					retval = libipv6addr_get_registry_string(&ipv6addr, temp2string);
 					if ( retval == 0 ) {
-						snprintf(tempstring, sizeof(tempstring), "%s.%s", temp2string, resultstring);
-						snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+						snprintf(tempstring, sizeof(tempstring) - 1, "%s.%s", temp2string, resultstring);
+						snprintf(resultstring, LINEBUFFER - 1, "%s", tempstring);
 					};
 				} else if ( (typeinfo & (IPV6_NEW_ADDR_6TO4 | IPV6_ADDR_MAPPED | IPV6_ADDR_COMPATv4)) != 0 ) {
 					/* fill IPv4 address */
@@ -457,21 +537,21 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 					retval = libipv4addr_get_registry_string(&ipv4addr, temp2string);
 					if ( retval == 0 ) {
 						/* IPv4 registry */
-						snprintf(tempstring, sizeof(tempstring), "%s.%s", temp2string, resultstring);
-						snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+						snprintf(tempstring, sizeof(tempstring) - 1, "%s.%s", temp2string, resultstring);
+						snprintf(resultstring, LINEBUFFER - 1, "%s", tempstring);
 					};
 				};
 			} else if (ipv4addr.flag_valid == 1) {
-				snprintf(resultstring, NI_MAXHOST, "ipv4-addr.addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "ipv4-addr.addrtype.ipv6calc");
 
 				retval = libipv4addr_get_registry_string(&ipv4addr, temp2string);
 				if ( retval == 0 ) {
 					/* IPv4 registry */
-					snprintf(tempstring, sizeof(tempstring), "%s.%s", temp2string, resultstring);
-					snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+					snprintf(tempstring, sizeof(tempstring) - 1, "%s.%s", temp2string, resultstring);
+					snprintf(resultstring, LINEBUFFER - 1, "%s", tempstring);
 				};
 			} else {
-				snprintf(resultstring, NI_MAXHOST, "reverse-lookup-successful.addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "reverse-lookup-successful.addrtype.ipv6calc");
 			};
 			break;
 
@@ -480,39 +560,54 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 				if (flag_skipunknown != 0) {
 					return (1);
 				} else {
-					snprintf(resultstring, NI_MAXHOST, "not-ipv6.ouitype.ipv6calc");
+					snprintf(resultstring, LINEBUFFER - 1, "not-ipv6.ouitype.ipv6calc");
 					return (0);
 				};
 		       	};
 
+			typeinfo = ipv6addr_gettype(&ipv6addr);
+
 			/* check whether address has a OUI ID */
-			if ( ( ipv6addr_gettype(&ipv6addr) & (IPV6_ADDR_LINKLOCAL | IPV6_ADDR_SITELOCAL | IPV6_NEW_ADDR_AGU | IPV6_NEW_ADDR_6BONE | IPV6_NEW_ADDR_6TO4)) == 0 )  {
+			if ( ( typeinfo & (IPV6_ADDR_LINKLOCAL | IPV6_ADDR_SITELOCAL | IPV6_NEW_ADDR_AGU | IPV6_NEW_ADDR_6BONE | IPV6_NEW_ADDR_6TO4 | IPV6_ADDR_ULUA)) == 0 )  {
 				if (flag_skipunknown != 0) {
 					return (1);
 				} else {
-					snprintf(resultstring, NI_MAXHOST, "unresolvable.ouitype.ipv6calc");
+					snprintf(resultstring, LINEBUFFER - 1, "unresolvable.ouitype.ipv6calc");
 					return (0);
 				};
 		       	};
 
 			if ((ipv6addr_getoctett(&ipv6addr, 8) & 0x02) != 0) {
-				retval = libieee_get_short_vendor_string(resultstring, ipv6addr_getoctett(&ipv6addr, 8) ^0x02, ipv6addr_getoctett(&ipv6addr, 9), ipv6addr_getoctett(&ipv6addr, 10) );
+				macaddr.addr[0] = ipv6addr_getoctett(&ipv6addr, 8) ^0x02;
+				macaddr.addr[1] = ipv6addr_getoctett(&ipv6addr, 9);
+				macaddr.addr[2] = ipv6addr_getoctett(&ipv6addr, 10);
+				macaddr.addr[3] = ipv6addr_getoctett(&ipv6addr, 13);
+				macaddr.addr[4] = ipv6addr_getoctett(&ipv6addr, 14);
+				macaddr.addr[5] = ipv6addr_getoctett(&ipv6addr, 15);
+
+				retval = libieee_get_short_vendor_string(resultstring, &macaddr);
 				if (retval != 0) {
 					if (flag_skipunknown != 0) {
 						return (1);
 					} else {
-						snprintf(resultstring, NI_MAXHOST, "unresolvable.ouitype.ipv6calc");
+						snprintf(resultstring, LINEBUFFER - 1, "unresolvable.ouitype.ipv6calc");
 						return (0);
 					};
 				};
 				if (strlen(resultstring) == 0) {
-					snprintf(resultstring, NI_MAXHOST, "unknown.ouitype.ipv6calc");
+					snprintf(resultstring, LINEBUFFER - 1, "unknown.ouitype.ipv6calc");
 				} else {
-					snprintf(tempstring, sizeof(tempstring), "%s.ouitype.ipv6calc", resultstring);
-					snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+					snprintf(tempstring, sizeof(tempstring) - 1, "%s.ouitype.ipv6calc", resultstring);
+					snprintf(resultstring, LINEBUFFER - 1, "%s", tempstring);
 				};
 			} else {
-				snprintf(resultstring, NI_MAXHOST, "local-scope.ouitype.ipv6calc");
+				if ( (typeinfo & IPV6_NEW_ADDR_6TO4_MICROSOFT) != 0 ) {
+					snprintf(resultstring, LINEBUFFER - 1, "6to4-microsoft.ouitype.ipv6calc");
+				} else if ( (typeinfo & IPV6_NEW_ADDR_ISATAP) != 0 ) {
+					snprintf(resultstring, LINEBUFFER - 1, "ISATAP.ouitype.ipv6calc");
+				} else {
+					snprintf(resultstring, LINEBUFFER - 1, "local-scope.ouitype.ipv6calc");
+				};
 			};
 			break;
 			
@@ -521,7 +616,7 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 				if (flag_skipunknown != 0) {
 					return (1);
 				} else {
-					snprintf(resultstring, NI_MAXHOST, "not-ipv6.ipv6addrtype.ipv6calc");
+					snprintf(resultstring, LINEBUFFER - 1, "not-ipv6.ipv6addrtype.ipv6calc");
 					return (0);
 				};
 		       	};
@@ -529,21 +624,23 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 			typeinfo = ipv6addr_gettype(&ipv6addr);
 
 		       	if ( (typeinfo & IPV6_ADDR_LINKLOCAL) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "link-local.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "link-local.ipv6addrtype.ipv6calc");
 			} else if ( (typeinfo & IPV6_ADDR_SITELOCAL) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "site-local.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "site-local.ipv6addrtype.ipv6calc");
+			} else if ( (typeinfo & IPV6_ADDR_ULUA) != 0 ) {
+				snprintf(resultstring, LINEBUFFER - 1, "unique-local.ipv6addrtype.ipv6calc");
 			} else if ( (typeinfo & IPV6_NEW_ADDR_6BONE) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "6bone-global.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "6bone-global.ipv6addrtype.ipv6calc");
 			} else if ( (typeinfo & IPV6_NEW_ADDR_6TO4) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "6to4-global.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "6to4-global.ipv6addrtype.ipv6calc");
 			} else if ( (typeinfo & IPV6_NEW_ADDR_PRODUCTIVE) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "productive-global.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "productive-global.ipv6addrtype.ipv6calc");
 			} else if ( (typeinfo & IPV6_ADDR_MAPPED) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "mapped-ipv4.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "mapped-ipv4.ipv6addrtype.ipv6calc");
 			} else if ( (typeinfo & IPV6_ADDR_COMPATv4) != 0 ) {
-				snprintf(resultstring, NI_MAXHOST, "compat-ipv4.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "compat-ipv4.ipv6addrtype.ipv6calc");
 			} else {
-				snprintf(resultstring, NI_MAXHOST, "unknown-ipv6.ipv6addrtype.ipv6calc");
+				snprintf(resultstring, LINEBUFFER - 1, "unknown-ipv6.ipv6addrtype.ipv6calc");
 			};
 			break;
 
@@ -552,6 +649,29 @@ static int converttoken(char *resultstring, const char *token, const long int ou
 			return (1);
 	};
 
+	/* use cache ? */
+	if (flag_nocache == 0) {
+		/* calculate pointer */
+		if (cache_lru_max < cache_lru_limit) {
+			cache_lru_last++;
+			cache_lru_max++;
+		} else {
+			if (cache_lru_last == cache_lru_limit) {
+				cache_lru_last = 1;
+			} else {
+				cache_lru_last++;
+			};
+		};
+
+		/* store key and value */
+		snprintf(cache_lru_key_token[cache_lru_last - 1], NI_MAXHOST - 1, token);
+		cache_lru_key_outputtype[cache_lru_last - 1] = outputtype;
+		snprintf(cache_lru_value[cache_lru_last - 1], NI_MAXHOST - 1, resultstring);
+		if (ipv6calc_debug & 0x4) {
+			fprintf(stderr, "LRU cache: fill line=%d key_token=%s key_outputtype=%lx value=%s\n", cache_lru_last - 1, cache_lru_key_token[cache_lru_last - 1], cache_lru_key_outputtype[cache_lru_last - 1], cache_lru_value[cache_lru_last - 1]);
+		};
+	};
+
 	return (0);
 };
 #undef DEBUG_function_name
