@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc/lib
  * File       : libipv4addr.c
- * Version    : $Id: libipv4addr.c,v 1.11 2003/06/15 12:12:54 peter Exp $
+ * Version    : $Id: libipv4addr.c,v 1.14 2005/09/15 12:32:21 peter Exp $
  * Copyright  : 2002-2003 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
  *
  * Information:
@@ -243,7 +243,7 @@ int addr_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	if ( cp != NULL ) {
 		i = atoi(cp);
 		if (i < 0 || i > 32 ) {
-			snprintf(resultstring, NI_MAXHOST, "Illegal prefix length: '%s'", cp);
+			snprintf(resultstring, NI_MAXHOST - 1, "Illegal prefix length: '%s'", cp);
 			retval = 1;
 			return (retval);
 		};
@@ -263,7 +263,7 @@ int addr_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 		};
 	};
 	if ( cpoints != 3 ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, given address '%s' is not valid (only %d dots)!", addronlystring, cpoints);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, given address '%s' is not valid (only %d dots)!", addronlystring, cpoints);
 		retval = 1;
 		return (retval);
 	};
@@ -278,7 +278,7 @@ int addr_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	
 	for ( i = 0; i <= 3; i++ ) {
 		if ( ( compat[i] < 0 ) || ( compat[i] > 255 ) )	{
-			snprintf(resultstring, NI_MAXHOST, "Error, given IPv4 address '%s' is not valid (%d on position %d)!", addronlystring, compat[i], i+1);
+			snprintf(resultstring, NI_MAXHOST - 1, "Error, given IPv4 address '%s' is not valid (%d on position %d)!", addronlystring, compat[i], i+1);
 			retval = 1;
 			return (retval);
 		};
@@ -289,7 +289,7 @@ int addr_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	};
 
 	if ( result != expecteditems ) {
-		snprintf(resultstring, NI_MAXHOST, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
 		retval = 1;
 		return (retval);
 	};
@@ -362,7 +362,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 	if ( cp != NULL ) {
 		i = atoi(cp);
 		if (i < 0 || i > 32 ) {
-			snprintf(resultstring, NI_MAXHOST, "Illegal prefix length: '%s'", cp);
+			snprintf(resultstring, NI_MAXHOST - 1, "Illegal prefix length: '%s'", cp);
 			retval = 1;
 			return (retval);
 		};
@@ -377,7 +377,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 
 	/* check length 3 */
 	if (strlen(addronlystring) != 8) {
-		snprintf(resultstring, NI_MAXHOST, "Error, given hesadecimal IPv4 address '%s' is not valid (not proper length)!", addronlystring);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, given hexadecimal IPv4 address '%s' is not valid (not proper length)!", addronlystring);
 		retval = 1;
 		return (retval);
 	};
@@ -396,7 +396,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 	
 	for ( i = 0; i <= 3; i++ ) {
 		if ( ( compat[i] < 0 ) || ( compat[i] > 255 ) )	{
-			snprintf(resultstring, NI_MAXHOST, "Error, given IPv4 address '%s' is not valid (%d on position %d)!", addronlystring, compat[i], i+1);
+			snprintf(resultstring, NI_MAXHOST - 1, "Error, given IPv4 address '%s' is not valid (%d on position %d)!", addronlystring, compat[i], i+1);
 			retval = 1;
 			return (retval);
 		};
@@ -407,7 +407,7 @@ int addrhex_to_ipv4addrstruct(const char *addrstring, char *resultstring, ipv6ca
 	};
 
 	if ( result != expecteditems ) {
-		snprintf(resultstring, NI_MAXHOST, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
 		retval = 1;
 		return (retval);
 	};
@@ -452,12 +452,12 @@ int libipv4addr_ipv4addrstruct_to_string(const ipv6calc_ipv4addr *ipv4addrp, cha
 	char tempstring[NI_MAXHOST];
 
 	/* address */
-	snprintf(tempstring, sizeof(tempstring), "%u.%u.%u.%u", (unsigned int) ipv4addr_getoctett(ipv4addrp, 0), (unsigned int) ipv4addr_getoctett(ipv4addrp, 1), (unsigned int) ipv4addr_getoctett(ipv4addrp, 2), (unsigned int) ipv4addr_getoctett(ipv4addrp, 3));
+	snprintf(tempstring, sizeof(tempstring) - 1, "%u.%u.%u.%u", (unsigned int) ipv4addr_getoctett(ipv4addrp, 0), (unsigned int) ipv4addr_getoctett(ipv4addrp, 1), (unsigned int) ipv4addr_getoctett(ipv4addrp, 2), (unsigned int) ipv4addr_getoctett(ipv4addrp, 3));
 
 	if ( (formatoptions & FORMATOPTION_machinereadable) != 0 ) {
-		snprintf(resultstring, NI_MAXHOST, "IPV4=%s", tempstring);
+		snprintf(resultstring, NI_MAXHOST - 1, "IPV4=%s", tempstring);
 	} else {
-		snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+		snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring);
 	};
 
 	/* netmaks */
@@ -480,8 +480,10 @@ int libipv4addr_ipv4addrstruct_to_string(const ipv6calc_ipv4addr *ipv4addrp, cha
  */
 #define DEBUG_function_name "libipv4calc/get_registry_string"
 int libipv4addr_get_registry_string(const ipv6calc_ipv4addr *ipv4addrp, char *resultstring) {
-	int i, match = -1;
+	int i;
+	int match = -1;
 	uint32_t match_mask = 0;
+	uint8_t  octet_msb;
 
 	uint32_t ipv4 = ipv4addr_getdword(ipv4addrp);
 	
@@ -489,34 +491,46 @@ int libipv4addr_get_registry_string(const ipv6calc_ipv4addr *ipv4addrp, char *re
 		fprintf(stderr, "%s: Given IPv4 address: %08x\n", DEBUG_function_name, (unsigned int) ipv4);
 	};
 
-	/* run through database array */
-	for (i = 0; i < (int) ( sizeof(dbipv4addr_assignment) / sizeof(dbipv4addr_assignment[0])); i++) {
-		if ( (ipv4 & dbipv4addr_assignment[i].ipv4mask) == dbipv4addr_assignment[i].ipv4addr ) {
-			/* ok, entry matches */
-			if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
-				fprintf(stderr, "%s: Found match number: %d\n", DEBUG_function_name, i);
-			};
+#define OPTIMIZED_LOOKUP 1
+#ifdef OPTIMIZED_LOOKUP
+	/* lookup in hint table for faster start */
+	octet_msb = ipv4addr_getoctett(ipv4addrp, 0);
 
-			/* have already found one */
-			if ( match != -1 ) {
-				if ( dbipv4addr_assignment[i].ipv4mask > match_mask ) {
-					/* wins */
-					if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
-						fprintf(stderr, "%s: Overwrite match number: %d (old: %d)\n", DEBUG_function_name, i, match);
+	if (dbipv4addr_assignment_hint[octet_msb].start != - 1) {
+		for (i = dbipv4addr_assignment_hint[octet_msb].start; i <= dbipv4addr_assignment_hint[octet_msb].end; i++) {
+#else
+		for (i = 0; i < (int) ( sizeof(dbipv4addr_assignment) / sizeof(dbipv4addr_assignment[0])); i++) {
+#endif
+			/* run through database array */
+			if ( (ipv4 & dbipv4addr_assignment[i].ipv4mask) == dbipv4addr_assignment[i].ipv4addr ) {
+				/* ok, entry matches */
+				if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+					fprintf(stderr, "%s: Found match number: %d\n", DEBUG_function_name, i);
+				};
+
+				/* have already found one */
+				if ( match != -1 ) {
+					if ( dbipv4addr_assignment[i].ipv4mask > match_mask ) {
+						/* this entry wins */
+						if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+							fprintf(stderr, "%s: Overwrite match number: %d (old: %d)\n", DEBUG_function_name, i, match);
+						};
+						match = i;
+						match_mask = dbipv4addr_assignment[i].ipv4mask;
+					} else {
+						if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
+							fprintf(stderr, "%s: No overwriting of match number: %d (candidate: %d)\n", DEBUG_function_name, match, i);
+						};
 					};
+				} else {
 					match = i;
 					match_mask = dbipv4addr_assignment[i].ipv4mask;
-				} else {
-					if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
-						fprintf(stderr, "%s: Overwrite match number: %d (old: %d)\n", DEBUG_function_name, i, match);
-					};
 				};
-			} else {
-				match = i;
-				match_mask = dbipv4addr_assignment[i].ipv4mask;
 			};
 		};
+#ifdef OPTIMIZED_LOOKUP
 	};
+#endif
 
 	if ( (ipv6calc_debug & DEBUG_libipv4addr) != 0 ) {
 		fprintf(stderr, "%s: Final match number: %d\n", DEBUG_function_name, match);
@@ -524,10 +538,10 @@ int libipv4addr_get_registry_string(const ipv6calc_ipv4addr *ipv4addrp, char *re
 	
 	/* result */
 	if ( match > -1 ) {
-		snprintf(resultstring, NI_MAXHOST, "%s", dbipv4addr_assignment[match].string_registry);
+		snprintf(resultstring, NI_MAXHOST - 1, "%s", dbipv4addr_assignment[match].string_registry);
 		return(0);
 	} else {
-		snprintf(resultstring, NI_MAXHOST, "%s", "unknown");
+		snprintf(resultstring, NI_MAXHOST - 1, "%s", "unknown");
 		return(1);
 	};
 };
@@ -608,15 +622,15 @@ int libipv4addr_to_reversestring(ipv6calc_ipv4addr *ipv4addrp, char *resultstrin
 			fprintf(stderr, "%s: bit: %d = noctett: %u, value: %x\n", DEBUG_function_name, nbit, noctett, octett);
 		};
 
-		snprintf(tempstring, sizeof(tempstring), "%s%u", resultstring, octett);
-		snprintf(resultstring, NI_MAXHOST, "%s.", tempstring);
+		snprintf(tempstring, sizeof(tempstring) - 1, "%s%u", resultstring, octett);
+		snprintf(resultstring, NI_MAXHOST - 1, "%s.", tempstring);
 	};
 
 	if (bit_start == 1) {
-		snprintf(tempstring, sizeof(tempstring), "%sin-addr.arpa.", resultstring);
+		snprintf(tempstring, sizeof(tempstring) - 1, "%sin-addr.arpa.", resultstring);
 	};
 
-	snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+	snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring);
 
 	if ( (formatoptions & FORMATOPTION_printuppercase) != 0 ) {
 		string_to_upcase(resultstring);
