@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc
  * File       : libipv6addr.c
- * Version    : $Id: libipv6addr.c,v 1.12 2003/08/15 11:58:28 peter Exp $
+ * Version    : $Id: libipv6addr.c,v 1.21 2005/09/15 12:32:21 peter Exp $
  * Copyright  : 2001-2002 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
  *
  * Information:
@@ -19,6 +19,7 @@
 #include "ipv6calctypes.h"
 #include "libipv6calc.h"
 #include "libipv6calcdebug.h"
+#include "../databases/ipv6-assignment/dbipv6addr_assignment.h"
 
 
 /*
@@ -235,11 +236,8 @@ void ipv6addr_clearall(ipv6calc_ipv6addr *ipv6addrp) {
  */
 #define DEBUG_function_name "libipv6addr/ipv6addr_copy"
 void ipv6addr_copy(ipv6calc_ipv6addr *ipv6addrp_dst, const ipv6calc_ipv6addr *ipv6addrp_src) {
-	int i;
 
-	for (i = 0; i < (int) sizeof(ipv6calc_ipv6addr); i++) {
-		*(ipv6addrp_dst + i) = *(ipv6addrp_src + i);
-	};
+	*(ipv6addrp_dst) = *(ipv6addrp_src);
 	
 	return;
 };
@@ -270,6 +268,12 @@ uint32_t ipv6addr_gettype(const ipv6calc_ipv6addr *ipv6addrp) {
 		return (type);
 	};
 
+	/* address space information  */
+	if ((st & 0xFC000000u) == 0xFC000000u) {
+		/* FC00::/7 -> Unique Local IPv6 Unicast Address */
+		type |= IPV6_ADDR_ULUA;
+	};
+
 	/* address space information  */
 	if ((st & 0xE0000000u) == 0x20000000u) {
 		/* 2000::/3 -> global unicast */
@@ -285,10 +289,20 @@ uint32_t ipv6addr_gettype(const ipv6calc_ipv6addr *ipv6addrp) {
 	if ((st & 0xFFFF0000u) == 0x20020000u) {
 		/* 2002::/16 -> 6to4 tunneling */
 		type |= IPV6_NEW_ADDR_6TO4;
+
+		if (	(ipv6addr_getword(ipv6addrp, 3) == 0) &&
+			(ipv6addr_getword(ipv6addrp, 4) == 0) &&
+			(ipv6addr_getword(ipv6addrp, 5) == 0) &&
+			(ipv6addr_getword(ipv6addrp, 6) == ipv6addr_getword(ipv6addrp, 1)) &&
+			(ipv6addr_getword(ipv6addrp, 7) == ipv6addr_getword(ipv6addrp, 2)) ) {
+			/* 2002:<ipv4addr>::<ipv4addr> -> usually Microsoft does this */
+			type |= IPV6_NEW_ADDR_6TO4_MICROSOFT;
+		};
 	};
 	
-	if ((st & 0xFFFF0000u) == 0x20010000u) {
-		/* 2001::/16 -> productive IPv6 address space */
+	if (!(type & (IPV6_NEW_ADDR_6BONE | IPV6_NEW_ADDR_6TO4)) && (st & 0xE0000000u) == 0x20000000u) {
+		/* 2000::/3 -> productive IPv6 address space */
+		/*  except 3ffe::/16 (6BONE) and 2002::/16 (6TO4) */
 		type |= IPV6_NEW_ADDR_PRODUCTIVE;
 	};
 	
@@ -306,8 +320,7 @@ uint32_t ipv6addr_gettype(const ipv6calc_ipv6addr *ipv6addrp) {
 	   000 and 111 as unicasts.
 	 */
 
-	/* original from kernel, only | and return changed */	
-	if ( ((st & 0xE0000000u) != 0x00000000u) && (st & 0xE0000000u) != 0xE0000000u) {
+	if ( (((st & 0xE0000000u) != 0x00000000u) && (st & 0xE0000000u) != 0xE0000000u) || ((st & 0xFC000000u) == 0xFC000000u)) {
 		type |= IPV6_ADDR_UNICAST;
 		return (type);
 	};
@@ -366,67 +379,42 @@ uint32_t ipv6addr_gettype(const ipv6calc_ipv6addr *ipv6addrp) {
 	return (type);
 };
 
-
 /*
- * Get IPv6 address assignement information
+ * get registry number of an IPv6 address
  *
- * See also: http://www.iana.org/assignments/ipv6-tla-assignments
- *
- * in : ipv6addrp = pointer to IPv6 address structure
+ * in:  ipv6addr = IPv6 address structure
  * out: assignment number (-1 = no result)
  */
+#define DEBUG_function_name "libipv6calc/getregistry"
 int ipv6addr_getregistry(const ipv6calc_ipv6addr *ipv6addrp) {
-	uint32_t st, st1, st2, st3;
+	char resultstring[NI_MAXHOST];
+	int i;
 
-	st =  ipv6addr_getdword(ipv6addrp, 0); /* 32 MSB */
-	st1 = ipv6addr_getdword(ipv6addrp, 1);
-	st2 = ipv6addr_getdword(ipv6addrp, 2);
-	st3 = ipv6addr_getdword(ipv6addrp, 3); /* 32 LSB */
-	
-	if ((st & 0xFFFF0000u) == 0x3FFE0000u) {
-		/* 3ffe::/16 -> 6bone space */
+	i = libipv6addr_get_registry_string(ipv6addrp, resultstring);
+
+	if (i != 0) {
+		return(IPV6_ADDR_REGISTRY_RESERVED);
+	};
+
+	if (strcmp(resultstring, "IANA") == 0) {
+		return(IPV6_ADDR_REGISTRY_IANA);
+	} else if (strcmp(resultstring, "APNIC") == 0) {
+		return(IPV6_ADDR_REGISTRY_APNIC);
+	} else if (strcmp(resultstring, "ARIN") == 0) {
+		return(IPV6_ADDR_REGISTRY_ARIN);
+	} else if (strcmp(resultstring, "RIPENCC") == 0) {
+		return(IPV6_ADDR_REGISTRY_RIPE);
+	} else if (strcmp(resultstring, "LACNIC") == 0) {
+		return(IPV6_ADDR_REGISTRY_LACNIC);
+	} else if (strcmp(resultstring, "6BONE") == 0) {
 		return(IPV6_ADDR_REGISTRY_6BONE);
+	} else if (strcmp(resultstring, "6TO4") == 0) {
+		return(-1);
+	} else {
+		return(-1);
 	};
-		
-	if ((st & 0xFFFF0000u) == 0x20010000u) {
-		/* 2001::/16 -> productive IPv6 address space */
-		
-		if ( (st & 0xFFFFFE00u) == 0x20010000u) {
-			/* 2001:0000::/23 -> IANA */
-			return(IPV6_ADDR_REGISTRY_IANA);
-		};
-		
-		if ( (st & 0xFFFFFE00u) == 0x20010200u ||
-		     (st & 0xFFFFFE00u) == 0x20010C00u ) {
-			/* 2001:0200::/23 -> APNIC */
-			/* 2001:0C00::/23 -> APNIC */
-			return(IPV6_ADDR_REGISTRY_APNIC);
-		};
-		
-		if ( (st & 0xFFFFFE00u) == 0x20010400u) {
-			/* 2001:0400::/23 -> ARIN */
-			return(IPV6_ADDR_REGISTRY_ARIN);
-		};
-		
-		if ( (st & 0xFFFFFE00u) == 0x20010600u ||
-		     (st & 0xFFFFFE00u) == 0x20010800u || 
-		     (st & 0xFFFFFE00u) == 0x20010A00u ) {
-			/* 2001:0600::/23 -> RIPE NCC */
-			/* 2001:0800::/23 -> RIPE NCC */
-			/* 2001:0A00::/23 -> RIPE NCC */
-			return(IPV6_ADDR_REGISTRY_RIPE);
-		};
-
-		if ( (st & 0xFFFFFE00u) == 0x20011200u ) {
-			/* 2001:1200::/23 -> LACNIC */
-			return(IPV6_ADDR_REGISTRY_LACNIC);
-		};
-
-		return (IPV6_ADDR_REGISTRY_RESERVED);
-	};
-
-	return(-1);
-};
+}
+#undef DEBUG_function_name
 
 
 /*
@@ -436,22 +424,38 @@ int ipv6addr_getregistry(const ipv6calc_ipv6addr *ipv6addrp) {
  * mod: resultstring
  * ret: 1=not found, 0=ok
  */
+#define DEBUG_function_name "libipv6calc/get_registry_string"
 int libipv6addr_get_registry_string(const ipv6calc_ipv6addr *ipv6addrp, char *resultstring) {
-	int i, registry;
+	int i;
+	int match = -1;
 
-	registry = ipv6addr_getregistry(ipv6addrp);
+	uint32_t ipv6 = ipv6addr_getdword(ipv6addrp, 0);
+	
+	if ( (ipv6calc_debug & DEBUG_libipv6addr) != 0 ) {
+		fprintf(stderr, "%s: Given ipv6 address: %08x\n", DEBUG_function_name, (unsigned int) ipv6);
+	};
 
-	if ( registry != -1 ) {
-		for ( i = 0; i < (int) (sizeof(ipv6calc_ipv6addrregistry) / sizeof(ipv6calc_ipv6addrregistry[0])); i++ ) {
-			if ( ipv6calc_ipv6addrregistry[i].number == registry ) {
-				snprintf(resultstring, NI_MAXHOST, "%s", ipv6calc_ipv6addrregistry[i].tokensimple);
-				return (0);
+	for (i = 0; i < (int) ( sizeof(dbipv6addr_assignment) / sizeof(dbipv6addr_assignment[0])); i++) {
+		/* run through database array */
+		if ( (ipv6 & dbipv6addr_assignment[i].ipv6mask) == dbipv6addr_assignment[i].ipv6addr ) {
+			/* ok, entry matches */
+			if ( (ipv6calc_debug & DEBUG_libipv6addr) != 0 ) {
+				fprintf(stderr, "%s: Found match\n", DEBUG_function_name);
 			};
+			match = i;
 		};
 	};
 
-	return (1);
+	/* result */
+	if ( match > -1 ) {
+		snprintf(resultstring, NI_MAXHOST - 1, "%s", dbipv6addr_assignment[match].string_registry);
+		return(0);
+	} else {
+		snprintf(resultstring, NI_MAXHOST - 1, "%s", "unknown");
+		return(1);
+	};
 };
+#undef DEBUG_function_name
 
 
 /*
@@ -484,7 +488,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 		return (1);
 	};
 
-	snprintf(tempstring, sizeof(tempstring), "%s", addrstring);
+	snprintf(tempstring, sizeof(tempstring) - 1, "%s", addrstring);
 	
 	ipv6addr_clearall(ipv6addrp);
 
@@ -510,7 +514,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	if ( cp != NULL ) {
 		i = atoi(cp);
 		if (i < 0 || i > 128 ) {
-			snprintf(resultstring, NI_MAXHOST, "Illegal prefix length: '%s'", cp);
+			snprintf(resultstring, NI_MAXHOST - 1, "Illegal prefix length: '%s'", cp);
 			retval = 1;
 			return (retval);
 		};
@@ -530,7 +534,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 			fprintf(stderr, "%s: Result of uncompressed string: '%s'\n", DEBUG_function_name, tempstring);
 		};
 		if ( result != 0 ) {
-			snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+			snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring);
 			retval = 1;
 			return (retval);
 		};
@@ -556,9 +560,9 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	};
 	if ( ! ( ( ( ccolons == 7 ) && ( cpoints == 0 ) ) ||  ( ( ccolons == 6 ) && ( cpoints == 3 ) ) ) ) {
 		if (strstr(addronlystring, "::")) {
-			snprintf(resultstring, NI_MAXHOST, "Error, given address expanded to '%s' is not valid!", tempstring);
+			snprintf(resultstring, NI_MAXHOST - 1, "Error, given address expanded to '%s' is not valid!", tempstring);
 		} else {
-			snprintf(resultstring, NI_MAXHOST, "Error, given address '%s' is not valid!", addrstring);
+			snprintf(resultstring, NI_MAXHOST - 1, "Error, given address '%s' is not valid!", addrstring);
 		};
 		retval = 1;
 		return (retval);
@@ -578,7 +582,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 		/* check compat */
 		for ( i = 0; i <= 3; i++ ) {
 			if ( compat[i] > 255 )	{
-				snprintf(resultstring, NI_MAXHOST, "Error, given compatv4/mapped address '%s' is not valid (%d)!", addrstring, compat[i]);
+				snprintf(resultstring, NI_MAXHOST - 1, "Error, given compatv4/mapped address '%s' is not valid (%d)!", addrstring, compat[i]);
 				retval = 1;
 				return (retval);
 			};
@@ -597,7 +601,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	};
 
 	if ( result != expecteditems ) {
-		snprintf(resultstring, NI_MAXHOST, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error splitting address %s, got %d items instead of %d!", addronlystring, result, expecteditems);
 		retval = 1;
 		return (retval);
 	};
@@ -605,7 +609,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 	/* check address words range */
 	for ( i = 0; i <= 7; i++ ) {
 		if ( (temp[i] < 0) || (temp[i] > 0xffff) )	{
-			snprintf(resultstring, NI_MAXHOST, "Error, given address '%s' is not valid on position %d (%x)!", addronlystring, i, temp[i]);
+			snprintf(resultstring, NI_MAXHOST - 1, "Error, given address '%s' is not valid on position %d (%x)!", addronlystring, i, temp[i]);
 			retval = 1;
 			return (retval);
 		};
@@ -637,7 +641,7 @@ int addr_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv6calc_
 		/* test, whether compatv4/mapped/ISATAP is really one */
 	/*
 		if ( (ipv6addrp->scope & (IPV6_ADDR_COMPATv4 | IPV6_ADDR_MAPPED | IPV6_NEW_ADDR_ISATAP)) == 0 ) {
-			snprintf(resultstring, NI_MAXHOST, "Error, given address '%s' is not valid compatv4/mapped/ISATAP one!", addrstring);
+			snprintf(resultstring, NI_MAXHOST - 1, "Error, given address '%s' is not valid compatv4/mapped/ISATAP one!", addrstring);
 			retval = 1;
 			return (retval);
 		};
@@ -669,23 +673,23 @@ static int ipv6addrstruct_to_uncompaddr(const ipv6calc_ipv6addr *ipv6addrp, char
 	/* print array */
 	if ( (ipv6addrp->scope & (IPV6_ADDR_COMPATv4 | IPV6_ADDR_MAPPED)) != 0 ) {
 		if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-			snprintf(tempstring, sizeof(tempstring), "%04x:%04x:%04x:%04x:%04x:%04x:%u.%u.%u.%u", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
+			snprintf(tempstring, sizeof(tempstring) - 1, "%04x:%04x:%04x:%04x:%04x:%04x:%u.%u.%u.%u", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 		} else {
-			snprintf(tempstring, sizeof(tempstring), "%x:%x:%x:%x:%x:%x:%u.%u.%u.%u", (unsigned int) ipv6addr_getword(ipv6addrp, 0), (unsigned int) ipv6addr_getword(ipv6addrp, 1), (unsigned int) ipv6addr_getword(ipv6addrp, 2), (unsigned int) ipv6addr_getword(ipv6addrp, 3), (unsigned int) ipv6addr_getword(ipv6addrp, 4), (unsigned int) ipv6addr_getword(ipv6addrp, 5), (unsigned int) ipv6addrp->in6_addr.s6_addr[12], (unsigned int) ipv6addrp->in6_addr.s6_addr[13], (unsigned int) ipv6addrp->in6_addr.s6_addr[14], (unsigned int) ipv6addrp->in6_addr.s6_addr[15]);
+			snprintf(tempstring, sizeof(tempstring) - 1, "%x:%x:%x:%x:%x:%x:%u.%u.%u.%u", (unsigned int) ipv6addr_getword(ipv6addrp, 0), (unsigned int) ipv6addr_getword(ipv6addrp, 1), (unsigned int) ipv6addr_getword(ipv6addrp, 2), (unsigned int) ipv6addr_getword(ipv6addrp, 3), (unsigned int) ipv6addr_getword(ipv6addrp, 4), (unsigned int) ipv6addr_getword(ipv6addrp, 5), (unsigned int) ipv6addrp->in6_addr.s6_addr[12], (unsigned int) ipv6addrp->in6_addr.s6_addr[13], (unsigned int) ipv6addrp->in6_addr.s6_addr[14], (unsigned int) ipv6addrp->in6_addr.s6_addr[15]);
 		};
 	} else {
 		if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-			snprintf(tempstring, sizeof(tempstring), "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
+			snprintf(tempstring, sizeof(tempstring) - 1, "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x", ipv6addr_getword(ipv6addrp, 0), ipv6addr_getword(ipv6addrp, 1), ipv6addr_getword(ipv6addrp, 2), ipv6addr_getword(ipv6addrp, 3), ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
 		} else {
-			snprintf(tempstring, sizeof(tempstring), "%x:%x:%x:%x:%x:%x:%x:%x", (unsigned int) ipv6addr_getword(ipv6addrp, 0), (unsigned int) ipv6addr_getword(ipv6addrp, 1), (unsigned int) ipv6addr_getword(ipv6addrp, 2), (unsigned int) ipv6addr_getword(ipv6addrp, 3), (unsigned int) ipv6addr_getword(ipv6addrp, 4), (unsigned int) ipv6addr_getword(ipv6addrp, 5), (unsigned int) ipv6addr_getword(ipv6addrp, 6), (unsigned int) ipv6addr_getword(ipv6addrp, 7));
+			snprintf(tempstring, sizeof(tempstring) - 1, "%x:%x:%x:%x:%x:%x:%x:%x", (unsigned int) ipv6addr_getword(ipv6addrp, 0), (unsigned int) ipv6addr_getword(ipv6addrp, 1), (unsigned int) ipv6addr_getword(ipv6addrp, 2), (unsigned int) ipv6addr_getword(ipv6addrp, 3), (unsigned int) ipv6addr_getword(ipv6addrp, 4), (unsigned int) ipv6addr_getword(ipv6addrp, 5), (unsigned int) ipv6addr_getword(ipv6addrp, 6), (unsigned int) ipv6addr_getword(ipv6addrp, 7));
 		};
 	};
 
 	if (ipv6addrp->flag_prefixuse == 1) {
 		/* append prefix length */
-		snprintf(resultstring, NI_MAXHOST, "%s/%u", tempstring, ipv6addrp->prefixlength);
+		snprintf(resultstring, NI_MAXHOST - 1, "%s/%u", tempstring, ipv6addrp->prefixlength);
 	} else {
-		snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+		snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring);
 	};
 
 	retval = 0;	
@@ -712,12 +716,12 @@ static int ipv6addrstruct_to_uncompaddrprefix(const ipv6calc_ipv6addr *ipv6addrp
 
 	/* test for misuse */
 	if ( ((ipv6addrp->scope & (IPV6_ADDR_COMPATv4 | IPV6_ADDR_MAPPED)) != 0 ) && (ipv6addrp->prefixlength > 96) ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, cannot print prefix of a compatv4/mapped address with prefix length bigger than 96!");
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, cannot print prefix of a compatv4/mapped address with prefix length bigger than 96!");
 		retval = 1;
 		return (retval);
 	};
 	if ( ipv6addrp->prefixlength == 0 ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, cannot print prefix of a address with prefix length 0!");
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, cannot print prefix of a address with prefix length 0!");
 		retval = 1;
 		return (retval);
 	};
@@ -728,21 +732,21 @@ static int ipv6addrstruct_to_uncompaddrprefix(const ipv6calc_ipv6addr *ipv6addrp
 	while (i <= max ) {
 		if ( i < max ) {
 			if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%04x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%04x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			} else {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			};
 		} else {
 			if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%04x", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%04x", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			} else {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%x", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%x", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			};
 		};
 		i++;
-		snprintf(tempstring1, sizeof(tempstring1), "%s", tempstring2);
+		snprintf(tempstring1, sizeof(tempstring1) - 1, "%s", tempstring2);
 	};
-	snprintf(resultstring, NI_MAXHOST, "%s", tempstring1);
+	snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring1);
 
 	if ( (ipv6calc_debug & DEBUG_libipv6addr) != 0 ) {
 		fprintf(stderr, "%s: result string: %s\n", DEBUG_function_name, resultstring);
@@ -773,12 +777,12 @@ static int ipv6addrstruct_to_uncompaddrsuffix(const ipv6calc_ipv6addr *ipv6addrp
 
 	/* test for misuse */
 	if ( ( (ipv6addrp->scope & (IPV6_ADDR_COMPATv4 | IPV6_ADDR_MAPPED)) != 0) && ( ipv6addrp->prefixlength > 96 ) ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, cannot print suffix of a compatv4/mapped address with prefix length bigger than 96!");
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, cannot print suffix of a compatv4/mapped address with prefix length bigger than 96!");
 		retval = 1;
 		return (retval);
 	};
 	if ( ipv6addrp->prefixlength == 128 ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, cannot print suffix of a address with prefix length 128!");
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, cannot print suffix of a address with prefix length 128!");
 		retval = 1;
 		return (retval);
 	};
@@ -788,25 +792,25 @@ static int ipv6addrstruct_to_uncompaddrsuffix(const ipv6calc_ipv6addr *ipv6addrp
 	tempstring1[0] = '\0';
 	while (i <= max ) {
 		if ( ( ( ipv6addrp->scope & (IPV6_ADDR_COMPATv4 | IPV6_ADDR_MAPPED)) != 0 ) && ( i == 6 ) ) {
-			snprintf(tempstring2, sizeof(tempstring2), "%s%u.%u.%u.%u", tempstring1, ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
+			snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%u.%u.%u.%u", tempstring1, ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 			i = max;
 		} else if ( i < max ) {
 			if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%04x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%04x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			} else {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%x:", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			};
 		} else {
 			if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%04x", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%04x", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			} else {
-				snprintf(tempstring2, sizeof(tempstring2), "%s%x", tempstring1, ipv6addr_getword(ipv6addrp, i));
+				snprintf(tempstring2, sizeof(tempstring2) - 1, "%s%x", tempstring1, ipv6addr_getword(ipv6addrp, i));
 			};
 		};
 		i++;
-		snprintf(tempstring1, sizeof(tempstring1), "%s", tempstring2);
+		snprintf(tempstring1, sizeof(tempstring1) - 1, "%s", tempstring2);
 	};
-	snprintf(resultstring, NI_MAXHOST, "%s", tempstring1);
+	snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring1);
 
 	if ( (ipv6calc_debug & DEBUG_libipv6addr) != 0 ) {
 		fprintf(stderr, "%s: result string: %s\n", DEBUG_function_name, resultstring);
@@ -963,7 +967,7 @@ int tokenlsb64_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv
 	};
 	
 	if ( strlen(addrstring) != 16 ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, given /token '%s' is not valid (length != 16!", addrstring);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, given /token '%s' is not valid (length != 16!", addrstring);
 		retval = 1;
 		return (retval);
 	};
@@ -971,13 +975,13 @@ int tokenlsb64_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv
 	/* scan address into array */
 	result = sscanf(addrstring, "%04x%04x%04x%04x", &temp[0], &temp[1], &temp[2], &temp[3]);
 	if ( result != 4 ) {
-		snprintf(resultstring, NI_MAXHOST, "Error splitting address '%s', got %d items instead of 4!", addrstring, result);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error splitting address '%s', got %d items instead of 4!", addrstring, result);
 		retval = 1;
 		return (retval);
 	};
 
 	/* set prefix */
-	snprintf(tempstring, sizeof(tempstring),  "0:0:0:0:%04x:%04x:%04x:%04x", temp[0], temp[1], temp[2], temp[3]);
+	snprintf(tempstring, sizeof(tempstring) - 1,  "0:0:0:0:%04x:%04x:%04x:%04x", temp[0], temp[1], temp[2], temp[3]);
 
 	/* store into structure */
 	retval = addr_to_ipv6addrstruct(tempstring, resultstring, ipv6addrp);
@@ -1006,13 +1010,13 @@ int identifier_to_ipv6addrstruct(const char *addrstring, char *resultstring, ipv
 	};
 	
 	if ( strlen(addrstring) > 19 ) {
-		snprintf(resultstring, NI_MAXHOST, "Error, given identifier identifier '%s' is too long (length > 16!", addrstring);
+		snprintf(resultstring, NI_MAXHOST - 1, "Error, given identifier identifier '%s' is too long (length > 16!", addrstring);
 		retval = 1;
 		return (retval);
 	};
 
 	/* set prefix */
-	snprintf(tempstring, sizeof(tempstring), "0:0:0:0:%s", addrstring);
+	snprintf(tempstring, sizeof(tempstring) - 1, "0:0:0:0:%s", addrstring);
 
 	/* store into structure */
 	retval = addr_to_ipv6addrstruct(tempstring, resultstring, ipv6addrp);
@@ -1037,7 +1041,7 @@ int libipv6addr_ipv6addrstruct_to_tokenlsb64(const ipv6calc_ipv6addr *ipv6addrp,
 	int retval = 1;
 	
 	/* print array */
-	snprintf(resultstring, NI_MAXHOST, "%04x%04x%04x%04x", ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
+	snprintf(resultstring, NI_MAXHOST - 1, "%04x%04x%04x%04x", ipv6addr_getword(ipv6addrp, 4), ipv6addr_getword(ipv6addrp, 5), ipv6addr_getword(ipv6addrp, 6), ipv6addr_getword(ipv6addrp, 7));
 
 	if ( (formatoptions & FORMATOPTION_printlowercase) != 0 ) {
 		/* nothing to do */
@@ -1068,12 +1072,12 @@ int libipv6addr_to_octal(const ipv6calc_ipv6addr *ipv6addrp, char *resultstring,
 	char tempstring[NI_MAXHOST];
 
 	if ( (formatoptions & FORMATOPTION_printfulluncompressed) != 0 ) {
-		snprintf(tempstring, sizeof(tempstring), "\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o", ipv6addrp->in6_addr.s6_addr[0], ipv6addrp->in6_addr.s6_addr[1], ipv6addrp->in6_addr.s6_addr[2], ipv6addrp->in6_addr.s6_addr[3], ipv6addrp->in6_addr.s6_addr[4], ipv6addrp->in6_addr.s6_addr[5], ipv6addrp->in6_addr.s6_addr[6], ipv6addrp->in6_addr.s6_addr[7], ipv6addrp->in6_addr.s6_addr[8], ipv6addrp->in6_addr.s6_addr[9], ipv6addrp->in6_addr.s6_addr[10], ipv6addrp->in6_addr.s6_addr[11], ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
+		snprintf(tempstring, sizeof(tempstring) - 1, "\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o\\%03o", ipv6addrp->in6_addr.s6_addr[0], ipv6addrp->in6_addr.s6_addr[1], ipv6addrp->in6_addr.s6_addr[2], ipv6addrp->in6_addr.s6_addr[3], ipv6addrp->in6_addr.s6_addr[4], ipv6addrp->in6_addr.s6_addr[5], ipv6addrp->in6_addr.s6_addr[6], ipv6addrp->in6_addr.s6_addr[7], ipv6addrp->in6_addr.s6_addr[8], ipv6addrp->in6_addr.s6_addr[9], ipv6addrp->in6_addr.s6_addr[10], ipv6addrp->in6_addr.s6_addr[11], ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 	} else {
-		snprintf(tempstring, sizeof(tempstring), "\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o", ipv6addrp->in6_addr.s6_addr[0], ipv6addrp->in6_addr.s6_addr[1], ipv6addrp->in6_addr.s6_addr[2], ipv6addrp->in6_addr.s6_addr[3], ipv6addrp->in6_addr.s6_addr[4], ipv6addrp->in6_addr.s6_addr[5], ipv6addrp->in6_addr.s6_addr[6], ipv6addrp->in6_addr.s6_addr[7], ipv6addrp->in6_addr.s6_addr[8], ipv6addrp->in6_addr.s6_addr[9], ipv6addrp->in6_addr.s6_addr[10], ipv6addrp->in6_addr.s6_addr[11], ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
+		snprintf(tempstring, sizeof(tempstring) - 1, "\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o\\%o", ipv6addrp->in6_addr.s6_addr[0], ipv6addrp->in6_addr.s6_addr[1], ipv6addrp->in6_addr.s6_addr[2], ipv6addrp->in6_addr.s6_addr[3], ipv6addrp->in6_addr.s6_addr[4], ipv6addrp->in6_addr.s6_addr[5], ipv6addrp->in6_addr.s6_addr[6], ipv6addrp->in6_addr.s6_addr[7], ipv6addrp->in6_addr.s6_addr[8], ipv6addrp->in6_addr.s6_addr[9], ipv6addrp->in6_addr.s6_addr[10], ipv6addrp->in6_addr.s6_addr[11], ipv6addrp->in6_addr.s6_addr[12], ipv6addrp->in6_addr.s6_addr[13], ipv6addrp->in6_addr.s6_addr[14], ipv6addrp->in6_addr.s6_addr[15]);
 	};
 
-	snprintf(resultstring, NI_MAXHOST, "%s", tempstring);
+	snprintf(resultstring, NI_MAXHOST - 1, "%s", tempstring);
 	retval = 0;	
 	return (retval);
 };
