@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc/lib
  * File       : libipv6addr.h
- * Version    : $Id: libipv6addr.h,v 1.13 2003/06/15 14:24:01 peter Exp $
+ * Version    : $Id: libipv6addr.h,v 1.17 2005/07/25 20:59:43 peter Exp $
  * Copyright  : 2001-2003 by Peter Bieringer <pb (at) bieringer.de> except the parts taken from kernel source
  *
  * Information:
@@ -30,6 +30,13 @@ typedef struct {
 	int flag_valid;			/* address structure filled */
 } ipv6calc_ipv6addr;
 
+/* IPv6 address assignment structure */
+typedef struct {
+	const uint32_t ipv6addr;
+	const uint32_t ipv6mask;
+	const char *string_registry;
+} s_ipv6addr_assignment;
+
 /* IPv6 address type definitions 
  * with credits to kernel and USAGI developer team
  * basic information was taken from "kernel/include/net/ipv6.h"
@@ -63,6 +70,8 @@ typedef struct {
 #define IPV6_ADDR_MAPPED			(uint32_t) 0x1000U
 #define IPV6_ADDR_RESERVED			(uint32_t) 0x2000U	/* reserved address space */
 
+#define IPV6_ADDR_ULUA				(uint32_t) 0x4000U	/* Unique Local Unicast Address */
+
 #define IPV6_NEW_ADDR_6TO4			(uint32_t) 0x00010000U
 #define IPV6_NEW_ADDR_6BONE			(uint32_t) 0x00020000U
 #define IPV6_NEW_ADDR_AGU			(uint32_t) 0x00040000U
@@ -70,6 +79,7 @@ typedef struct {
 #define IPV6_NEW_ADDR_SOLICITED_NODE		(uint32_t) 0x00100000U
 #define IPV6_NEW_ADDR_ISATAP			(uint32_t) 0x00200000U
 #define IPV6_NEW_ADDR_PRODUCTIVE		(uint32_t) 0x00400000U
+#define IPV6_NEW_ADDR_6TO4_MICROSOFT		(uint32_t) 0x00800000U
 
 /* Registries */
 #define IPV6_ADDR_REGISTRY_6BONE	0x01
@@ -79,6 +89,7 @@ typedef struct {
 #define IPV6_ADDR_REGISTRY_RIPE		0x05
 #define IPV6_ADDR_REGISTRY_LACNIC	0x06
 #define IPV6_ADDR_REGISTRY_RESERVED	0x0e
+#define IPV6_ADDR_REGISTRY_UNKNOWN	0x0f
 
 /* text representations */
 typedef struct {
@@ -97,13 +108,15 @@ typedef struct {
 	{ IPV6_ADDR_COMPATv4		, "compat-v4" },
 	{ IPV6_ADDR_MAPPED		, "mapped" },
 	{ IPV6_ADDR_RESERVED		, "reserved" },
+	{ IPV6_ADDR_ULUA		, "unique-local-unicast" },
 	{ IPV6_NEW_ADDR_6TO4		, "6to4" },
 	{ IPV6_NEW_ADDR_6BONE		, "6bone" },
 	{ IPV6_NEW_ADDR_AGU		, "global-unicast" },
 	{ IPV6_NEW_ADDR_UNSPECIFIED	, "unspecified" },
 	{ IPV6_NEW_ADDR_SOLICITED_NODE	, "solicited-node" },
 	{ IPV6_NEW_ADDR_ISATAP		, "ISATAP" },
-	{ IPV6_NEW_ADDR_PRODUCTIVE	, "productive" }
+	{ IPV6_NEW_ADDR_PRODUCTIVE	, "productive" },
+	{ IPV6_NEW_ADDR_6TO4_MICROSOFT	, "6to4-microsoft" }
 };
 
 typedef struct {
@@ -118,7 +131,8 @@ typedef struct {
 	{ IPV6_ADDR_REGISTRY_APNIC	, "APNIC"	, "APNIC" 	},
 	{ IPV6_ADDR_REGISTRY_ARIN	, "ARIN" 	, "ARIN"	},
 	{ IPV6_ADDR_REGISTRY_RIPE	, "RIPE NCC"	, "RIPENCC"	},
-	{ IPV6_ADDR_REGISTRY_RESERVED	, "reserved"	, "reserved"	}
+	{ IPV6_ADDR_REGISTRY_RESERVED	, "reserved"	, "reserved"	},
+	{ IPV6_ADDR_REGISTRY_UNKNOWN	, "unknown"	, "unknown"	}
 };
 #endif
 
