@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc/lib
  * File       : libipv6calc.c
- * Version    : $Id: libipv6calc.c,v 1.10 2003/06/15 16:15:07 peter Exp $
+ * Version    : $Id: libipv6calc.c,v 1.16 2005/09/15 12:32:21 peter Exp $
  * Copyright  : 2001-2003 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
@@ -99,34 +99,34 @@ void string_to_reverse_dotted(char *string) {
 	ptrptr = &cptr;
 
 	/* clear result string */
-	snprintf(resultstring, sizeof(resultstring), "%s", "");
+	snprintf(resultstring, sizeof(resultstring) - 1 , "%s", "");
 
 	/* check for starting dot */
 	if ( string[0] == '.' ) {
-		snprintf(tempstring, sizeof(tempstring), "%s.", resultstring);
-		snprintf(resultstring, sizeof(resultstring), "%s", tempstring);
+		snprintf(tempstring, sizeof(tempstring) - 1, "%s.", resultstring);
+		snprintf(resultstring, sizeof(resultstring) - 1, "%s", tempstring);
 	};
 
 	token = strtok_r(string, ".", ptrptr);
 
 	while (token != NULL) {
 		if (flag_first == 1) {
-			snprintf(tempstring, sizeof(tempstring), "%s%s", token, resultstring);
+			snprintf(tempstring, sizeof(tempstring) - 1, "%s%s", token, resultstring);
 			flag_first = 0;
 		} else {
-			snprintf(tempstring, sizeof(tempstring), "%s.%s", token, resultstring);
+			snprintf(tempstring, sizeof(tempstring) - 1, "%s.%s", token, resultstring);
 		};
-		snprintf(resultstring, sizeof(resultstring), "%s", tempstring);
+		snprintf(resultstring, sizeof(resultstring) - 1, "%s", tempstring);
 
 		token = strtok_r(NULL, ".", ptrptr);
 	};
 	
 	if ( string[strlen(string) - 1] == '.' ) {
-		snprintf(tempstring, sizeof(tempstring), ".%s", resultstring);
-		snprintf(resultstring, sizeof(resultstring), "%s", tempstring);
+		snprintf(tempstring, sizeof(tempstring) - 1, ".%s", resultstring);
+		snprintf(resultstring, sizeof(resultstring) - 1, "%s", tempstring);
 	};
 
-	snprintf(string, NI_MAXHOST, ".%s", resultstring);
+	snprintf(string, NI_MAXHOST - 1, ".%s", resultstring);
 	
 	return;
 };
@@ -158,8 +158,8 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		if (string[i] == ':') { numcolons++; };
 		if (string[i] == '-') { numdashes++; };
 		if (string[i] == ' ') { numspaces++; };
-		if (isdigit(string[i])) { numdigits++; };
-		if (isxdigit(string[i])) { numxdigits++; };
+		if (isdigit((int) string[i])) { numdigits++; };
+		if (isxdigit((int) string[i])) { numxdigits++; };
 	};
 
 	if ( (ipv6calc_debug & DEBUG_libipv6calc) != 0 ) {
@@ -170,7 +170,7 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		fprintf(stderr, "%s:  numspaces :%d\n", DEBUG_function_name, numspaces);
 		fprintf(stderr, "%s:  numdigits :%d\n", DEBUG_function_name, numdigits);
 		fprintf(stderr, "%s:  numxdigits:%d\n", DEBUG_function_name, numxdigits);
-		fprintf(stderr, "%s:  length    :%d\n", DEBUG_function_name, length);
+		fprintf(stderr, "%s:  length    :%d\n", DEBUG_function_name, (int) length);
 	};
 
 	if ( length == 20 && numdots == 0 && numcolons == 0 ) {
@@ -217,7 +217,7 @@ uint32_t libipv6calc_autodetectinput(const char *string) {
 		/* Check whether minimum 1 xdigit is between colons */
 		j = 0;
 		for (i = 0; i < (int) length; i++) {
-			if (isxdigit(string[i])) {
+			if (isxdigit((int) string[i])) {
 				j++;
 				if ( j > 2 ) {
 					/* more than 2 xdigits */
