@@ -1,7 +1,7 @@
 /*
  * Project    : ipv6calc
  * File       : librfc1924.c
- * Version    : $Id: librfc1924.c,v 1.5 2002/04/05 20:55:10 peter Exp $
+ * Version    : $Id: librfc1924.c,v 1.7 2005/09/15 12:32:21 peter Exp $
  * Copyright  : 2001-2002 by Peter Bieringer <pb (at) bieringer.de>
  *
  * Information:
@@ -283,7 +283,7 @@ int ipv6addrstruct_to_base85(const ipv6calc_ipv6addr *ipv6addrp, char *resultstr
 	};
 
 	/* convert */	
-	snprintf(resultstring, NI_MAXHOST, "%s", pk85(bv) );	
+	snprintf(resultstring, NI_MAXHOST - 1, "%s", pk85(bv) );	
 
    	retval = 0;	
 	return (retval);
@@ -362,14 +362,14 @@ int librfc1924_formatcheck(const char *string, char *infostring) {
 
 	/* check length */
 	if ( length != 20 ) {
-		snprintf(infostring, NI_MAXHOST, "Given base85 formatted address has not 20 chars!");
+		snprintf(infostring, NI_MAXHOST - 1, "Given base85 formatted address has not 20 chars!");
 		return (1);
 	};
 
 	/* check for base85 chars only content */
 	cnt = strspn(string, librfc1924_charset);
 	if ( cnt != 20 ) {
-		snprintf(infostring, NI_MAXHOST, "Illegal character in given base85 formatted address on position %d (%c)!", (int) cnt + 1, string[cnt]);
+		snprintf(infostring, NI_MAXHOST - 1, "Illegal character in given base85 formatted address on position %d (%c)!", (int) cnt + 1, string[cnt]);
 		return (1);
 	};
 
